/*
 *    CFM to MachO Framework Bridge Routines
 *
 *		by Shigeyuki Hirai, 06/02/10
 */


OSStatus initEntryPoints (CFStringRef framework, CFBundleRef *outRef);
void deallocEntryPoints (CFBundleRef *inRef);
void * getProcAddress (CFStringRef proc, CFBundleRef *inRef);

// -------------------------

OSStatus initEntryPoints (CFStringRef framework, CFBundleRef *ref)
{
	OSStatus err = noErr;
	FSRef frameworksFolderRef;
	CFURLRef baseURL = nil;
	CFURLRef bundleURL = nil;

	
	err = FSFindFolder (kOnAppropriateDisk, kFrameworksFolderType, true, &frameworksFolderRef); // find framework
	if (noErr != err) {
        DebugStr ("\pCould not find frameworks folder");
		return err;
	}
	baseURL = CFURLCreateFromFSRef(kCFAllocatorSystemDefault, &frameworksFolderRef);
	if (!baseURL) {
        DebugStr ("\pCould create specific Framework base URL");
		return err;
    }
	bundleURL = CFURLCreateCopyAppendingPathComponent(kCFAllocatorSystemDefault, baseURL, framework, false);
	if (!bundleURL) {
        DebugStr ("\pCould create specific Framework bundle URL");
		return paramErr;
    }
	*ref = CFBundleCreate (kCFAllocatorSystemDefault, bundleURL); // create ref to a framework bundle
	if (!*ref) {
        DebugStr ("\pCould not create specific Framework bundle");
		return paramErr;
	}
    if (!CFBundleLoadExecutable (*ref)) { // If the code was successfully loaded, look for our function.
    	DebugStr ("\pCould not load MachO executable");
    	return paramErr;
	}
	if (err != noErr && *ref != nil) {
		CFRelease(*ref);
		*ref = nil;
	}
	if (bundleURL != nil) {
		CFRelease(bundleURL);
	}	
	if (baseURL != nil) {
		CFRelease(baseURL);
	}	
	return err;
}

// -------------------------

void deallocEntryPoints (CFBundleRef *ref)
{
	if (*ref != NULL) {
        CFBundleUnloadExecutable (*ref); //	Unload the bundle's executable code. 
        CFRelease (*ref);
        *ref = NULL;
	}
}

// -------------------------

void * getProcAddress (CFStringRef proc, CFBundleRef *inRef)
{
	return CFBundleGetFunctionPointerForName (*inRef, proc);
}

